# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'client.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainClientWindow(object):
    def setupUi(self, MainClientWindow):
        MainClientWindow.setObjectName("MainClientWindow")
        MainClientWindow.resize(730, 575)
        MainClientWindow.setMinimumSize(QtCore.QSize(730, 575))
        self.centralwidget = QtWidgets.QWidget(MainClientWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.label_contacts = QtWidgets.QLabel(self.centralwidget)
        self.label_contacts.setGeometry(QtCore.QRect(10, 7, 251, 16))
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_contacts.setFont(font)
        self.label_contacts.setObjectName("label_contacts")
        self.label_history = QtWidgets.QLabel(self.centralwidget)
        self.label_history.setGeometry(QtCore.QRect(270, 7, 441, 16))
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_history.setFont(font)
        self.label_history.setObjectName("label_history")
        self.label_new_message = QtWidgets.QLabel(self.centralwidget)
        self.label_new_message.setGeometry(QtCore.QRect(270, 377, 441, 16))
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_new_message.setFont(font)
        self.label_new_message.setObjectName("label_new_message")
        self.list_contacts = QtWidgets.QListView(self.centralwidget)
        self.list_contacts.setGeometry(QtCore.QRect(10, 30, 250, 450))
        self.list_contacts.setObjectName("list_contacts")
        self.list_messages = QtWidgets.QListView(self.centralwidget)
        self.list_messages.setGeometry(QtCore.QRect(270, 30, 450, 335))
        self.list_messages.setObjectName("list_messages")
        self.text_message = QtWidgets.QTextEdit(self.centralwidget)
        self.text_message.setGeometry(QtCore.QRect(270, 400, 450, 80))
        self.text_message.viewport().setProperty(
            "cursor", QtGui.QCursor(QtCore.Qt.IBeamCursor))
        self.text_message.setObjectName("text_message")
        self.btn_add_contact = QtWidgets.QPushButton(self.centralwidget)
        self.btn_add_contact.setGeometry(QtCore.QRect(9, 487, 122, 36))
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setKerning(True)
        self.btn_add_contact.setFont(font)
        self.btn_add_contact.setCursor(
            QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.btn_add_contact.setObjectName("btn_add_contact")
        self.btn_remove_contact = QtWidgets.QPushButton(self.centralwidget)
        self.btn_remove_contact.setGeometry(QtCore.QRect(139, 487, 122, 36))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.btn_remove_contact.setFont(font)
        self.btn_remove_contact.setCursor(
            QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.btn_remove_contact.setObjectName("btn_remove_contact")
        self.btn_clear = QtWidgets.QPushButton(self.centralwidget)
        self.btn_clear.setGeometry(QtCore.QRect(469, 487, 122, 36))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.btn_clear.setFont(font)
        self.btn_clear.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.btn_clear.setObjectName("btn_clear")
        self.btn_send = QtWidgets.QPushButton(self.centralwidget)
        self.btn_send.setGeometry(QtCore.QRect(599, 487, 122, 36))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.btn_send.setFont(font)
        self.btn_send.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.btn_send.setObjectName("btn_send")
        MainClientWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainClientWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 730, 21))
        self.menubar.setObjectName("menubar")
        self.menu = QtWidgets.QMenu(self.menubar)
        self.menu.setObjectName("menu")
        self.menu_2 = QtWidgets.QMenu(self.menubar)
        self.menu_2.setObjectName("menu_2")
        MainClientWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainClientWindow)
        self.statusbar.setObjectName("statusbar")
        MainClientWindow.setStatusBar(self.statusbar)
        self.menu_exit = QtWidgets.QAction(MainClientWindow)
        self.menu_exit.setObjectName("menu_exit")
        self.menu_add_contact = QtWidgets.QAction(MainClientWindow)
        self.menu_add_contact.setObjectName("menu_add_contact")
        self.menu_del_contact = QtWidgets.QAction(MainClientWindow)
        self.menu_del_contact.setObjectName("menu_del_contact")
        self.menu.addAction(self.menu_exit)
        self.menu_2.addAction(self.menu_add_contact)
        self.menu_2.addAction(self.menu_del_contact)
        self.menubar.addAction(self.menu.menuAction())
        self.menubar.addAction(self.menu_2.menuAction())

        self.retranslateUi(MainClientWindow)
        QtCore.QMetaObject.connectSlotsByName(MainClientWindow)

    def retranslateUi(self, MainClientWindow):
        _translate = QtCore.QCoreApplication.translate
        MainClientWindow.setWindowTitle(
            _translate("MainClientWindow", "Чат программа Alpha-релиз"))
        self.label_contacts.setText(
            _translate("MainClientWindow", "Список контактов:"))
        self.label_history.setText(
            _translate("MainClientWindow", "История сообщений:"))
        self.label_new_message.setText(
            _translate("MainClientWindow", "Введите новое сообщение:"))
        self.btn_add_contact.setText(
            _translate("MainClientWindow", "+ Добавить"))
        self.btn_remove_contact.setText(
            _translate("MainClientWindow", "- Удалить"))
        self.btn_clear.setText(_translate("MainClientWindow", "Очистить"))
        self.btn_send.setText(_translate("MainClientWindow", "Отправить"))
        self.menu.setTitle(_translate("MainClientWindow", "Файл"))
        self.menu_2.setTitle(_translate("MainClientWindow", "Контакты"))
        self.menu_exit.setText(_translate("MainClientWindow", "Выход"))
        self.menu_add_contact.setText(
            _translate("MainClientWindow", "Добавить контакт"))
        self.menu_del_contact.setText(
            _translate("MainClientWindow", "Удалить контакт"))
